/*
 * Iz񃉃Cu
 */

#ifndef MINATO__VECTOR_H
#define MINATO__VECTOR_H

#define BOOL int
#define TRUE 1
#define FALSE 0

/* IzIuWFNg */
typedef struct {
   void** mTable;
   int mTableSize;
   
   int mCount;
} vector_obj;

//////////////////////////////////////////////////////////////////////
// ֐`
//////////////////////////////////////////////////////////////////////
// 
vector_obj* vector_new(int first_size);
#define VECTOR_NEW(size) vector_new(size)

void vector_delete(vector_obj* self);                           // 

vector_obj* vector_clone(vector_obj* v);                        // N[

#define vector_item(o, i) ((vector_obj*)o)->mTable[i]

//void* vector_item(vector_obj* self, int index);                 // ACeԂ
/*
//////////////////////////////////////////////////////////////////////
// ACeԂ
//////////////////////////////////////////////////////////////////////
void* vector_item(vector_obj* self, int index)
{
    if(index >= 0 && index < self->mCount) {
        return self->mTable[index];
    }
    else {
        return NULL;
    }
}
   
///////////////////////////////////////////////////////////////////////
// TCYԂ
///////////////////////////////////////////////////////////////////////
int vector_size(vector_obj* self)
{
   return self->mCount;
}
int vector_size(vector_obj* self);                              // TCYԂ
*/

#define vector_size(o) ((vector_obj*)o)->mCount

int vector_index(vector_obj* self, void* item);                 // ACẽCfbNXԂ

void vector_exchange(vector_obj* self, int n, void* item);      // ACen̈ʒuɒuȂ
void vector_add(vector_obj* self, void* item);                  // ACeŌɒǉ
void vector_insert(vector_obj* self, int n, void* item);        // ACen̈ʒuɑ}
void* vector_erase(vector_obj* self, int n);                    // n̈ʒũACe폜
void* vector_pop_back(vector_obj* self);                        // Ō̃ACeo
void* vector_pop_front(vector_obj* self);                       // ŏ̃ACeo
void vector_clear(vector_obj* self);                            // ACeNA

typedef int (*sort_if)(void* left, void* right);             // -1ŃG[
BOOL vector_sort(vector_obj* self, sort_if fun);                // ACe\[gFALSEԂCTRL-Cꂽ



#endif
